/*
 * pcallang.h - language-dependent strings (month and day names, option file 
 * keywords, preprocessor tokens, prepositions, etc.):
 *
 * Revision history:
 *
 *	4.1	AWR	08/16/91	support -G flag (outlined gray)
 *
 *	4.02	AWR	07/02/91	support -v flag (version number);
 *					add ordinal_suffix(); add format
 *					specifiers to help message
 *
 *	4.0	AWR	03/01/91	expand parameter message to explain
 *					parameter meaning when -w specified
 *
 *		AWR	02/19/91	revise ordinal definitions for
 *					support of negative ordinals
 *
 *		AWR	02/06/91	add text describing expression syntax
 *
 *		AWR	02/04/91	support "odd" and "even" ordinals
 *
 *		AWR	01/28/91	support -B (blank fill squares) flag
 *					and -O (outline "gray" dates) flag
 *
 *		AWR	01/16/91	added moon file support (tokens, help
 *					file text, error messages); define
 *					note block heading here
 *
 *		AWR	01/07/91	added support for -w (whole year) flag
 *
 *	3.0	AWR	12/10/90	added support for "workday", "weekday",
 *					"holiday", et. al.
 *
 *		AWR	11/15/90	extracted from pcal.c; revised
 *					to contain all language-dependent
 *					strings
 */

#define ALL	"all"		/* command-line or date file keyword */

#ifdef MAIN_MODULE

char *months[12] = {
	"January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December"
	};

/* Must be a 2-D array so address within may be used as an initializer;
 * wildcard names must be in same order as symbolic names in pcaldefs.h
 */
char days[14][12] = {
        "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
        "Saturday", 		/* weekday names */
	"day", "weekday", "workday", "holiday", "nonweekday", "nonworkday",
	"nonholiday"		/* wildcards */
	};

/* preprocessor tokens: token name, token code, dispatch routine */

KWD_F pp_info[] = {
	"define",	PP_DEFINE,	do_define,
	"else",		PP_ELSE,	NULL,
	"endif",	PP_ENDIF,	NULL,
	"ifdef",	PP_IFDEF,	do_ifdef,
	"ifndef",	PP_IFNDEF,	do_ifndef,
	"include",	PP_INCLUDE,	NULL,		/* do_include */
	"undef",	PP_UNDEF,	do_undef,
	NULL,		PP_OTHER,	NULL };		/* must be last */

/* ordinal numbers - e.g. "first Monday in September": ordinal name,
 * ordinal code, ordinal value; note that "all" is parsed as a keyword
 * and (depending on context) may be subsequently treated as an ordinal
 */

KWD_O ordinals[] = {
	"first",	ORD_POSNUM,	 1,
	"second",	ORD_POSNUM,	 2,
	"third",	ORD_POSNUM,	 3,
	"fourth",	ORD_POSNUM,	 4,
	"fifth",	ORD_POSNUM,	 5,
	"last",		ORD_NEGNUM,	-1,
	"odd",		ORD_ODD,	 0,
	"even",		ORD_EVEN,	 0,
	NULL,		ORD_OTHER,	 0 };		/* must be last */

/* allowable suffixes for ordinal numbers */

char *ord_suffix[] = { "st", "nd", "rd", "th", NULL };

/* prepositions - e.g., "Friday after fourth Thursday in November" */

KWD preps[] = {
	"before",	PR_BEFORE,
	"preceding",	PR_BEFORE,
	"on_or_before",	PR_ON_BEFORE,
	"oob",		PR_ON_BEFORE,
	"after",	PR_AFTER,
	"following",	PR_AFTER,
	"on_or_after",	PR_ON_AFTER,
	"ooa",		PR_ON_AFTER,
	NULL,		PR_OTHER };	/* must be last */

/* other keywords */

KWD keywds[] = {
	ALL,	 	DT_ALL,
	"each",		DT_ALL,
	"every",	DT_ALL,
	"note",		DT_NOTE,
	"opt",		DT_OPT,
	"year",		DT_YEAR,
	NULL,		DT_OTHER };	/* must be last */

/* moon phases (for moon file) */

KWD phases[] = {
	"NM",		MOON_NM,	/* new moon */
	"1Q",		MOON_1Q,	/* first quarter */
	"FQ",		MOON_1Q,	
	"FM",		MOON_FM,	/* full moon */
	"3Q",		MOON_3Q,	/* third (last) quarter */
	"LQ",		MOON_3Q,
	NULL,		MOON_OTHER };	/* must be last */

#else
extern char *months[];
extern char days[14][12];
extern KWD_F pp_info[];
extern KWD preps[];
extern KWD_O ordinals[];
extern char *ord_suffix[];
extern KWD keywds[];
extern KWD phases[];
#endif

/* minimum size of abbreviations  */

#define MIN_DAY_LEN	3
#define MIN_MONTH_LEN	3
#define MIN_PPTOK_LEN	3
#define MIN_PREP_LEN	7	/* distinguish "on_or_before", "on_or_after" */
#define MIN_ORD_LEN	3	/* distinguish "Thursday" from "third" */


/*
 * Symbolic names for command-line flags.  These may be changed
 * as desired in order to be meaningful in languages other than
 * English.
 */

#define F_INITIALIZE	'I'		/* re-initialize program defaults */
#define	F_BLACK_DAY	'b'		/* print day in black */
#define F_GRAY_DAY	'g'		/* print day in gray */

#define F_DAY_FONT	'd'		/* select alternate day font */
#define F_NOTES_FONT	'n'		/* select alternate notes font */
#define F_TITLE_FONT	't'		/* select alternate title font */

#define F_EMPTY_CAL	'e'		/* print empty calendar */
#define F_DATE_FILE	'f'		/* select alternate date file */
#define F_OUT_FILE	'o'		/* select alternate output file */

#define F_LANDSCAPE	'l'		/* landscape mode */
#define F_PORTRAIT	'p'		/* portrait mode */

#define F_HELP		'h'		/* generate full help message */
#define F_VERSION	'v'		/* generate version ID */

#define F_MOON_4	'm'		/* print new/half/full moons */
#define F_MOON_ALL	'M'		/* print all moons */

#define F_DEFINE	'D'		/* define preprocessor symbol */
#define F_UNDEF		'U'		/* undefine preprocessor symbol */

#define F_L_FOOT	'L'		/* define left foot string */
#define F_C_FOOT	'C'		/* define center foot string */
#define F_R_FOOT	'R'		/* define right foot string */

#define F_FIRST_DAY	'F'		/* define alternate starting day */

#define F_USA_DATES	'A'		/* parse American date format */
#define F_EUR_DATES	'E'		/* parse European date format */

#define F_X_TRANS	'X'		/* X-axis transformation */
#define F_Y_TRANS	'Y'		/* Y-axis transformation */
#define F_X_SCALE	'x'		/* X-axis scale factor */
#define F_Y_SCALE	'y'		/* Y-axis scale factor */

#define F_JULIAN	'j'		/* print Julian day (day of year) */
#define F_JULIAN_ALL	'J'		/* print Julian day and days left */

#define F_WHOLE_YEAR	'w'		/* print whole year per page */
					/* (cf. W_WYFLAG below) */

#define F_BLANK_BOXES	'B'		/* don't fill unused boxes */

#define F_OUTLINE_GRAY	'G'		/* outline and fill "gray" dates */
#define F_OUTLINE	'O'		/* draw "gray" dates as outlines */

/*
 * Flag usage information - not strictly language-dependent, but here anyway
 * (N.B.: all flags must be represented by an entry in this table!)
 *
 * Flags may appear in any of three places: in environment variable
 * PCAL_OPTS, on the command line, or in "opt" lines in the date file.
 * The command line is actually parsed twice: once before reading the date
 * file to get the flags needed in processing it (-e, -f, -b, -g, -D, -U, -A,
 * -E), and again after reading the date file to give the user one last
 * chance to override any of the other flags set earlier.  (Note, however,
 * that the only way to turn off -J|-j [Julian dates], -M|-m [moons], -w
 * [whole year], or -G|-O [outline "gray" dates] once selected is to use -I
 * to reinitialize all program defaults.)
 *
 * The table below supplies the following information about each flag:
 *
 *	- Its name (cf. symbolic definitions above)
 *
 *	- Whether or not it can take an (optional) argument
 *
 *	- Which passes parse it: P_ENV (environment variable), P_CMD1
 *	  (first command line pass), P_OPT ("opt" lines in date file), 
 *	  and P_CMD2 (second command line pass)
 *
 */

#ifdef MAIN_MODULE

FLAG_USAGE flag_tbl[] = {

/*	flag name	arg?	passes where parsed		*/

	F_INITIALIZE,	FALSE,	P_ENV | P_CMD1 | P_OPT | P_CMD2 ,

	F_BLACK_DAY,	TRUE,	P_ENV | P_CMD1 | P_OPT          ,
	F_GRAY_DAY,	TRUE,	P_ENV | P_CMD1 | P_OPT          ,

	F_DAY_FONT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_NOTES_FONT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_TITLE_FONT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,

	F_EMPTY_CAL,	FALSE,	P_ENV | P_CMD1                  ,
	F_DATE_FILE,	TRUE,	P_ENV | P_CMD1                  ,
	F_OUT_FILE,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,

	F_LANDSCAPE,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,
	F_PORTRAIT,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,

	F_HELP,		FALSE,	        P_CMD1                  ,
	F_VERSION,	FALSE,	        P_CMD1                  ,

	F_MOON_4,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,
	F_MOON_ALL,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,

	F_DEFINE,	TRUE,	P_ENV | P_CMD1                  ,
	F_UNDEF,	TRUE,	P_ENV | P_CMD1                  ,

	F_L_FOOT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_C_FOOT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_R_FOOT,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,

	F_FIRST_DAY,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,

	F_USA_DATES,	FALSE,	P_ENV | P_CMD1 | P_OPT          ,
	F_EUR_DATES,	FALSE,	P_ENV | P_CMD1 | P_OPT          ,

	F_X_TRANS,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_Y_TRANS,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_X_SCALE,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,
	F_Y_SCALE,	TRUE,	P_ENV          | P_OPT | P_CMD2 ,

	F_JULIAN,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,
	F_JULIAN_ALL,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,

	F_WHOLE_YEAR,	FALSE,	P_ENV | P_CMD1 | P_OPT          ,

	F_BLANK_BOXES,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,

	F_OUTLINE_GRAY,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,
	F_OUTLINE,	FALSE,	P_ENV          | P_OPT | P_CMD2 ,

	'-',		FALSE,	P_ENV | P_CMD1 | P_OPT | P_CMD2 ,
	'\0',		FALSE,	P_ENV | P_CMD1 | P_OPT | P_CMD2		/* must be last */
	};

#else
extern FLAG_USAGE flag_tbl[];
#endif

/*
 * Words used in usage() message - translate as necessary
 */

#define W_DEFAULT	"default"		/* translate as required */
#define W_USAGE		"Usage"

#define W_FONT		"FONT"			/* names of metavariables */
#define W_DAY		"DAY"
#define W_STRING	"STRING"
#define W_FILE		"FILE"
#define W_SYMBOL	"SYMBOL"
#define W_VALUE		"VALUE"

#define W_MM		"MM"			/* abbrev. for month, year */
#define W_YY		"YY"

#define W_WYFLAG	"-w"			/* must conform to F_WHOLE_YEAR */

#define W_BLACK		"black"			/* cf. color_msg() */
#define W_GRAY		"gray"


/* special flag_msg[] entries for end of option group, etc. */

#define END_GROUP	'\n', NULL, NULL, NULL		/* end of option group */
#define END_LIST	'\0', NULL, NULL, NULL		/* end of list */
#define GROUP_DEFAULT	' ', NULL, " "			/* group default */

/*
 * Message strings to be printed by usage() - translate as necessary
 */
#ifdef MAIN_MODULE

FLAG_MSG flag_msg[] = {

/*	flag name	metasyntax	description						default */

	F_INITIALIZE,	NULL,		"initialize all parameters to program defaults",	NULL,
	END_GROUP,

	F_BLACK_DAY,	W_DAY,		"print weekday in black",				NULL,
	F_GRAY_DAY,	W_DAY,		"print weekday in gray (see below)",			NULL,
	END_GROUP,

	F_OUTLINE_GRAY,	NULL,		"print \"gray\" dates as filled outlines",		NULL,
	F_OUTLINE,	NULL,		"print \"gray\" dates as unfilled outlines",		NULL,
#if NUM_STYLE == GRAY_NUMS
	GROUP_DEFAULT,										"gray characters",
#else
#if NUM_STYLE == OUTLINE_NUMS
	GROUP_DEFAULT,										"unfilled outlines",
#else
	GROUP_DEFAULT,										"filled outlines",
#endif
#endif

	END_GROUP,

	F_DAY_FONT,	W_FONT,		"specify alternate day name font",			DAYFONT,
	F_NOTES_FONT,	W_FONT,		"specify alternate notes font",				NOTESFONT,
	F_TITLE_FONT,	W_FONT,		"specify alternate title font",				TITLEFONT,
	END_GROUP,

	F_EMPTY_CAL,	NULL,		"generate empty calendar (ignore date file)",		NULL,
	END_GROUP,

	F_DATE_FILE,	W_FILE,		"specify alternate date file",				DATEFILE,
	END_GROUP,

#ifdef DEFAULT_OUTFILE
	F_OUT_FILE,	W_FILE,		"specify alternate output file",			DEFAULT_OUTFILE,
#else
	F_OUT_FILE,	W_FILE,		"specify alternate output file",			"stdout",
#endif
	END_GROUP,

	F_LANDSCAPE,	NULL,		"generate landscape-style calendar",			NULL,
	F_PORTRAIT,	NULL,		"generate portrait-style calendar",			NULL,
#if ROTATE == LANDSCAPE
	GROUP_DEFAULT,										"landscape",
#else	
	GROUP_DEFAULT,										"portrait",
#endif
	END_GROUP,

	F_HELP,		NULL,		"print this help message",				NULL,
	F_VERSION,	NULL,		"print version information",				NULL,
	END_GROUP,

	F_MOON_4,	NULL,		"draw a \"moon\" icon at full/new/half moons",		NULL,
	F_MOON_ALL,	NULL,		"draw a \"moon\" icon every day",			NULL,
#if DRAW_MOONS == NO_MOONS
	GROUP_DEFAULT,										"no moons",
#else
#if DRAW_MOONS == SOME_MOONS
	GROUP_DEFAULT,										"full/new/half moons",
#else
	GROUP_DEFAULT,										"every day",
#endif
#endif
	END_GROUP,

	F_DEFINE,	W_SYMBOL,	"define preprocessor symbol",				NULL,
	F_UNDEF,	W_SYMBOL,	"undefine preprocessor symbol",				NULL,
	END_GROUP,

	F_L_FOOT,	W_STRING,	"specify left foot string",				LFOOT,
	F_C_FOOT,	W_STRING,	"specify center foot string",				CFOOT,
	F_R_FOOT,	W_STRING,	"specify right foot string",				RFOOT,
	END_GROUP,

	F_FIRST_DAY,	W_DAY,		"specify starting day of week",				days[FIRST_DAY],
	END_GROUP,

	F_USA_DATES,	NULL,		"parse American dates (\"mm/dd{/yy}\" and \"month dd\")", NULL,
	F_EUR_DATES,	NULL,		"parse European dates (\"dd/mm{/yy}\" and \"dd month\")", NULL,
#if DATE_STYLE == USA_DATES
	GROUP_DEFAULT,										"American",
#else
	GROUP_DEFAULT,										"European",
#endif
	END_GROUP,

	F_X_TRANS,	W_VALUE,	"specify x-axis translation",				XTVAL,
	F_Y_TRANS,	W_VALUE,	"specify y-axis translation",				YTVAL,
	F_X_SCALE,	W_VALUE,	"specify x-axis scale factor",				XSVAL,
	F_Y_SCALE,	W_VALUE,	"specify y-axis scale factor",				YSVAL,
	END_GROUP,

	F_JULIAN,	NULL,		"print Julian day (day of year)",			NULL,
	F_JULIAN_ALL,	NULL,		"print Julian day and days left in year",		NULL,
#if JULIAN_DATES == NO_JULIANS
	GROUP_DEFAULT,										"neither",
#else
#if JULIAN_DATES == SOME_JULIANS
	GROUP_DEFAULT,										"Julian day",
#else
	GROUP_DEFAULT,										"both",
#endif
#endif
	END_GROUP,

	F_WHOLE_YEAR,	NULL,		"print whole year (12 consecutive months) per page",	NULL,
	END_GROUP,

	F_BLANK_BOXES,	NULL,		"leave unused boxes blank",				NULL,

	END_GROUP,			/* must precede END_LIST */

	END_LIST			/* must be last */
};

#else
extern FLAG_MSG flag_msg[];
#endif

/* Numeric parameter descriptions and text */

#ifdef MAIN_MODULE

#if __STDC__
PARAM_MSG param_msg[] = {
	W_YY,			"generate calendar for year " W_YY " (19" W_YY " if " W_YY " < 100)",
	W_MM " " W_YY,		"generate calendar for month " W_MM " (Jan = 1), year " W_YY,
	W_MM " " W_YY " N",	"generate calendars for N months, starting at " W_MM "/" W_YY,
	"(" W_DEFAULT ")",	"generate calendar for current month and/or year",
	"",			"",
	"if " W_WYFLAG " specified:",	"",
	"",			"",
	W_YY,			"generate calendar for year " W_YY " (19" W_YY " if " W_YY " < 100)",
	W_MM " " W_YY,		"generate calendars for 12 months, starting at " W_MM "/" W_YY,
	W_MM " " W_YY " N",	"generate calendars for N months, starting at " W_MM "/" W_YY,
	"",			"  (N rounded up to next multiple of 12)",
	"(" W_DEFAULT ")",	"generate calendar for current year",
	NULL,		NULL		/* must be last */
};
#else
PARAM_MSG param_msg[] = {
	"YY",		"generate calendar for year YY (19YY if YY < 100)",
	"MM YY",	"generate calendar for month MM (Jan = 1), year YY",
	"MM YY N",	"generate calendars for N months, starting at MM/YY",
	"(default)",	"generate calendar for current month and year",
	"",		"",
	"if -w specified:",	"",
	"",		"",
	"YY",		"generate calendar for year YY (19YY if YY < 100)",
	"MM YY",	"generate calendar for 12 months, starting at MM/YY",
	"MM YY N",	"generate calendars for N months, starting at MM/YY",
	"",		"  (N rounded up to next multiple of 12)",
	"(default)",	"generate calendar for current year",
	NULL,		NULL		/* must be last */
};
#endif

#else
extern PARAM_MSG param_msg[];
#endif

#define PARAM_MSGS	3	/* number of above to print in command-line syntax message */

/* Date file syntax message - lines are copied intact */

#ifdef MAIN_MODULE

char *date_msg[] = {
	"",
	"Date file syntax:",
	"",
	"The following rules describe the syntax of date file entries:",
	"",
	"  year <year>",
	"",
	"  opt <options>",
	"",
	"  note <month_spec> <text>",
	"  note <month> <text>",
	"",
	"  if -A flag (American date formats) specified:",
	"    <month_name> <day>{*} {<text>}",
	"    <month><sep><day>{<sep><year>}{*} {<text>}",
	"",
	"  if -E flag (European date formats) specified:",
	"    <day> <month_name>{*} {<text>}",
	"    <day><sep><month>{<sep><year>}{*} {<text>}",
	"",
	"  <ordinal> <day_name> in <month_spec>{*} {<text>}",
	"  <day_name> <prep> <date_spec>",
	"",
	"where",
	"",
	"  {x}          means x is optional",
	"",
	"  <date_spec> := any of the above date specs (not year, note, or opt)",
	"  <month_name> := first 3+ characters of name of month, or \"all\"",
	"  <month_spec> := <month_name>, or \"year\"",
	"  <day_name> := first 3+ characters of name of weekday, \"day\",",
	"                \"weekday\", \"workday\", \"holiday\", \"nonweekday\",",
	"                \"nonworkday\", or \"nonholiday\"",
	"  <ordinal> := ordinal number (\"1st\", \"2nd\", etc.), \"first\" .. \"fifth\",",
	"                \"last\", \"even\", \"odd\", or \"all\"",
	"  <prep> := \"before\", \"preceding\", \"after\", \"following\", \"on_or_before\",",
	"                or \"on_or_after\"",
	"  <sep> := one or more non-numeric, non-space, non-'*' characters",
	"  <month>, <day>, <year> are the numeric forms",
	"",
	"  <options> := any command-line option except -e, -f, -h, -v, -D, -U",
	"",
	"Comments start with '#' and run through end-of-line.",
	"",
	"Holidays may be flagged by specifying '*' as the last character of",
	"the date field(s), e.g. \"10/12* Columbus Day\", \"July 4* Independence",
	"Day\", etc.  Any dates flagged as holidays will be printed in gray, and",
	"any associated text will appear adjacent to the date.",
	"",
	"Note that the numeric date formats (mm/dd{/yy}, dd.mm{.yy}) support",
	"an optional year, which will become the subsequent default year.  The",
	"alphabetic date formats (month dd, dd month) do not support a year",
	"field; the \"year yy\" command is provided to reset the default year.",
	"",
	"\"Floating\" days may be specified in the date file as \"first Mon in ",
	"Sep\", \"last Mon in May\", \"4th Thu in Nov\", etc.; any word may be",
	"used in place of \"in\".  \"Relative floating\" days (e.g. \"Fri after 4th ",
	"Thu in Nov\") are also accepted; they may span month/year bounds.",
	"Pcal also accepts date specs such as \"all Friday{s} in October\", \"last",
	"Thursday in all\", etc., and produces the expected results; \"each\" and",
	"\"every\" are accepted as synonyms for \"all\".  Negative ordinals are",
	"allowed; \"-2nd\" means \"next to last\".",
	"",
	"The words \"day\", \"weekday\", \"workday\", and \"holiday\" may be used as",
	"wildcards: \"day\" matches any day, \"weekday\" matches any day normally",
	"printed in black, \"workday\" matches any day normally printed in black",
	"and not explicitly flagged as a holiday, and \"holiday\" matches any",
	"day explicitly flagged as a holiday.  \"Nonweekday\", \"nonworkday\",",
	"and \"nonholiday\" are also supported and have the obvious meanings.",
	"",
	"\"Odd\" and \"even\" do not refer to the actual date; instead, \"odd\"",
	"means \"alternate, starting with the first\"; \"even\" means \"alternate,",
	"starting with the second\".  Thus, \"odd Fridays in March\" refers to",
	"the first, third, and (if present) fifth Fridays in March - not to",
	"those Fridays falling on odd dates.",
	"",
	"\"All\" refers to each individual month; \"year\" refers to the year",
	"as an entity.  Thus \"odd Fridays in all\" refers to the first/third/",
	"fifth Friday of each month, while \"odd Fridays in year\" refers to",
	"the first Friday of January and every other Friday thereafter.",
	"",
	"Additional notes may be propagated to an empty calendar box by the",
	"inclusion of one or more lines of the form \"note <month> <text>\",",
	"where <month> may be numeric or alphabetic; \"note all <text>\"",
	"propagates <text> to each month in the current year.",
	"",
	"Pcal also allows format specifiers in the text (and foot strings -",
	"cf. the -L, -C, and -R options); each will be replaced by its",
	"equivalent string as outlined in the table below.  (Most of these are",
	"derived from the strftime() function; %[lo0+-] are Pcal-specific.)",
	"",
	"        %a : abbreviated weekday",
	"        %A : full weekday",
	"        %b : abbreviated month name",
	"        %B : full month name",
	"        %d : day of month (1-31)",
	"        %j : day of year (1-366)",
	"        %l : days left in year (0-365)",
	"        %m : month (1-12)",
	"        %U : week number (0-53)",
	"        %W : week number (0-53)",
	"        %y : year w/o century (00-99)",
	"        %Y : year w/century",
	"        %% : '%' character",
	"",
	"        %o : print number as ordinal",
	"        %0 : print number with leading zeroes",
	"        %+ : use following month or year",
	"        %- : use previous month or year",
	"",
	"%U considers the first logical Sunday (the first day of the week as",
	"printed; cf. the -F flag) of the year as the first day of week 1;",
	"%W uses the first logical Monday instead.  This is an extension of",
	"strftime()'s behavior.",
	"",
	"%o prints a number as an ordinal, with the appropriate suffix (\"st\",",
	"\"nd\", \"rd\", or \"th\" in English) appended; for example, \"%od\" prints",
	"the day of the month as \"1st\", \"2nd\", \"3rd\", etc.",
	"",
	"Unlike strftime(), Pcal's default is to print numbers (except %y)",
	"without leading zeroes.  If leading zeroes are desired, the '0'",
	"prefix may be used; for example, \"%0j\" prints the day of year as",
	"001-365.",
	"",
	"%+ and %- direct Pcal to substitute the following/previous month/year",
	"in the following [bBmyY] specifier; for example, %+B prints the name",
	"of the next month.",
	"",
	"Simple cpp-like functionality is provided.  The date file may include",
	"the following commands, which work like their cpp counterparts:",
	"",
	"        define <sym>",
	"        undef <sym>",
	"",
	"        if{n}def <expr>",
	"           ...",
	"        { else",
	"           ... }",
	"        endif",
	"",
	"        include <file>",
	"",
	"Note that these do not start with '#', which is reserved as a comment",
	"character.",
	"",
	"<sym> is a symbol name consisting of a letter followed by zero or",
	"more letters, digits, or underscores ('_').  Symbol names are always",
	"treated in a case-insensitive manner.",
	"",
	"<expr> is an expression consisting of symbol names joined by the logical",
	"operators (in order of precedence, high to low) '!' (unary negate), '&'",
	"(and), '^' (exclusive or), and '|' (inclusive or).  '&&' and '||' are",
	"accepted as synonyms for '&' and '|' respectively; the order of",
	"evaluation may be altered by the use of parentheses.  A symbol whose",
	"name is currently defined evaluates to TRUE; one whose name is not",
	"currently defined evaluates to FALSE.  Thus \"ifdef A | B | C\" is TRUE",
	"if any of the symbols A, B, and C is currently defined, and",
	"\"ifdef A & B & C\" is TRUE if all of them are.",
	"",
	"\"ifndef A | B | C\" is equivalent to \"ifdef !(A | B | C)\" (or, using",
	"DeMorgan's Law, \"ifdef !A & !B & !C\") - in other words, TRUE if none of",
	"the symbols A, B, and C is currently defined.",
	"",
	"\"define\" alone deletes all the current definitions; \"ifdef\" alone is",
	"always false; \"ifndef\" alone is always true.",
	"",
	"The file name in the \"include\" directive may optionally be surrounded",
	"by \"\" or <>.  In any case, path names are taken to be relative to",
	"the location of the file containing the \"include\" directive.",
	"",
	"",
	"Moon file syntax:",
	"",
	"The user may enter the dates and (optionally) times of quarter",
	"moons (from a reliable source such as an almanac or astronomical",
	"table) into a file called .moonXX (moonXX.dat on VMS), where XX is",
	"the last two digits of the year.  If such a file exists (in the",
	"same directory as the date file), Pcal will interpolate the phase",
	"of the moon from the information in this file instead of using the",
	"default algorithm.",
	"",
	"(Pcal originally used an extremely simplistic moon phase algorithm;",
	"the moon file was added to v4.0 to enable Pcal to interpolate the",
	"phase of the moon from the [presumably more accurate] information",
	"within.  More recently, the original moon phase algorithm was",
	"superseded by an astronomer-strength version, largely obviating",
	"the need for a moon file; however, it will continue to be",
	"supported for the foreseeable future.)",
	"",
	"Entries in the moon file must conform to the following syntax:",
	"",
	"  if -A flag (American date formats) specified:",
	"    <quarter> <month><sep><day> {<hour><sep><min>}",
	"",
	"  if -E flag (European date formats) specified:",
	"    <quarter> <day><sep><month> {<hour><sep><min>}",
	"",
	"where",
	"",
	"  <quarter> := \"nm\", \"fq\" or \"1q\", \"fm\", \"lq\" or \"3q\" (new",
	"                moon, first quarter, full moon, last quarter)",
	"  <hour>    := number 0-23 (24-hour clock)",
	"  <min>     := number 0-59",
	"",
	"This file must contain entries for all quarter moons in the year,",
	"in chronological order; if any errors are encountered, Pcal will",
	"revert to using its default algorithm.",
	"",
	"As in the date file, comments start with '#' and run through",
	"end-of-line.  ",
	NULL
	};
#else
extern char *date_msg[];
#endif

/* format strings for color_msg() - translate as necessary */
#define COLOR_MSG_1	"all days in %s"
#define COLOR_MSG_2	"in %s; others in %s"

/* format string for short usage() message */
#define USAGE_MSG	"\"%s -%c\" prints full description of flags, parameters, and file formats\n"

/* format strings for comment in PostScript output file */
#define VERSION_MSG	"Generated by %s %s"
#define DATEFILE_MSG	" from %s"

#define NOTES_HDR	"Notes"		/* title of "notes" box */
#define LINE_SEP	".p"		/* text line separator */

/* strings used in error messages */
#define ENV_VAR		"environment variable "
#define DATE_FILE	"date file "

/* Error and information messages - translate as necessary */

/* program error messages */
#define	E_ALLOC_ERR	"%s: calloc() failed - out of memory\n"
#define	E_FOPEN_ERR	"%s: can't open file %s\n"
#define	E_ILL_LINE	"%s: %s in file %s, line %d\n"
#define	E_ILL_MONTH	"%s: month %d not in range %d .. %d\n"
#define	E_ILL_OPT	"%s: unrecognized flag %s"
#define E_ILL_OPT2	" (%s\"%s\")"
#define	E_ILL_YEAR	"%s: year %d not in range %d .. %d\n"
#define	E_SYMFULL	"%s: symbol table full - can't define %s\n"
#define	E_UNT_IFDEF	"%s: unterminated if{n}def..{else..}endif in file %s\n"
#define E_FLAG_IGNORED	"%s: -%c flag ignored (%s\"%s\")\n"

/* preprocessor error strings */
#define E_ELSE_ERR	"unmatched \"else\""
#define E_END_ERR	"unmatched \"endif\""
#define E_GARBAGE	"extraneous data on \"%s\" line"
#define E_INV_DATE	"invalid date (or no match for wildcard)"
#define E_INV_LINE	"unrecognized line"
#define E_NESTING	"maximum file nesting level exceeded"
#define E_EXPR_SYNTAX	"syntax error in expression"

/* moon file error strings */
#define E_DATE_SEQ	"date or phase out of sequence"
#define E_PREM_EOF	"premature EOF"

/* information message (VMS, Amiga only) */
#define	I_OUT_NAME	"%s: output is in file %s\n"


/*
 * Yes, some source code in a header file.  This is a language-dependent
 * routine used by writefil.c to translate the %o (ordinal suffix) format
 * specifier into the appropriate string for printing, and seemed to belong
 * here with the other language dependencies.
 */
#ifdef WRITEFIL			/* include only in writefil.c */
#ifdef PROTOS
static char *ordinal_suffix(int num)
#else
static char *ordinal_suffix(num)
	int num;		/* ordinal number - assumed positive */
#endif
{
	static char buf[10];
	static char *suffix[] = { "th", "st", "nd", "rd" };
	int tens, units;

	tens = (num / 10) % 10;		/* select suffix */
	units = num % 10;
	strcpy(buf, suffix[(units > 3 || tens == 1) ? 0 : units]);

	return buf;
}
#endif
